#!/usr/bin/env python3

import requests
import json
from prettytable import PrettyTable

from .config import Config
from .functions import printError, printWarn

class Contacts(object):

    def __init__(self, config):
        self.config = config
        self.contacts = None
        self.format = 'table'

        self.table = PrettyTable()
        self.table.field_names = ['ID', 'Name', 'Email', 'Phone', 'Method']
        self.table.align['ID'] = 'l'
        self.table.align['Name'] = 'l'
        self.table.align['Email'] = 'l'
        self.table.align['Phone'] = 'l'
        self.table.align['Method'] = 'c'

    def fetchData(self):
        """Retrieve the list of all contacts"""

        # if data is already downloaded, use cached data
        if self.contacts != None:
            return True

        # check if headers are correctly set for authorization
        if not self.config.headers():
            return False

        if self.config.debug:
            print('GET', self.config.endpoint + 'contacts?', self.config.params())

        # Make request to API endpoint
        response = requests.get(self.config.endpoint + 'contacts', params=self.config.params(), headers=self.config.headers())

        # Check status code of response
        if response.status_code == 200:
            # Get list of contacts from response
            self.contacts = response.json()['contacts']
            return True
        else:
            printError('An error occurred:', response.status_code)
            self.contacts = None
            return False

    def list(self, id: str = '', name: str = '', email: str = '', phone: str = '', sort: str = '', reverse: bool = False, limit: int = 0):
        """Iterate through list of contacts and print details"""

        if self.fetchData():

            # if JSON was requested and no filters, then just print it without iterating through
            if (self.format == 'json' and not (id or name or email or phone or limit > 0)):
                print(json.dumps(self.contacts, indent=4))
                return

            for contact in self.contacts:
                if (id or name or email or phone):
                    if (id and 'id' in contact and contact['id'] == id) \
                        or (name and 'name' in contact and contact['name'] == name) \
                        or (email and 'email' in contact and contact['email'] == email) \
                        or (phone and 'phonenumber' in contact and contact['phonenumber'] == phone):
                        self.print(contact)
                else:
                    self.print(contact)

            self.printFooter(sort=sort, reverse=reverse, limit=limit)

    def add(self, name: str, email: str = '', sms: str = ''):
        """Add a contact for the given name"""

        if name and self.fetchData():

            for contact in self.contacts:
                if contact['name'] == name:
                    print(name, 'already exists and will not be added')
                    return False

            # Make request to API endpoint
            data = {
                'name': name,
                'channels': {
                    'email': email,
                    'sms': sms
                }
            }

            if self.config.debug:
                print('POST', self.config.endpoint + 'contacts?', data)

            if self.config.readonly:
                return False

            response = requests.post(self.config.endpoint + 'contacts',  data=json.dumps(data), headers=self.config.headers())

            # Check status code of response
            if response.status_code == 200:
                print('Added contact:', name)
                return True
            else:
                printError('Failed to add contact', name, 'with response code:', response.status_code)
                return False

        else:
            return False

    def remove(self, id: str = '', name: str = '', email: str = '', phone: str = ''):
        """Remove the contact for the given name"""

        if (id or name or email or phone) and self.fetchData():
            for contact in self.contacts:
                curr_id = contact['id']
                curr_name = contact['name']
                curr_email = contact['email'] if 'email' in contact else ''
                curr_phone = contact['phonenumber'] if 'phonenumber' in contact else ''

                if (id == curr_id) \
                    or (name and name == curr_name) \
                    or (email and email == curr_email) \
                    or (phone and phone == curr_phone):

                    if self.config.debug:
                        print('DELETE', self.config.endpoint + 'contacts/' + curr_id)

                    if self.config.readonly:
                        return False

                    # Make request to API endpoint
                    response = requests.delete(self.config.endpoint + 'contact/' + curr_id, headers=self.config.headers())

                    # Check status code of response
                    if response.status_code == 204:
                        print('Removed contact:', curr_name, '[', curr_id, ']')
                        return True
                    else:
                        printError('Failed to remove contact', curr_name, '[', curr_id, '] with response code:', response.status_code)
                        return False

        printWarn('No contact with given pattern found: id=' + id, 'name=' + name, 'email=' + email, 'phone=' + phone)

    def printFooter(self, sort: str = '', reverse: bool = False, limit: int = 0):
        """Print table if table format requested"""

        if (self.format == 'table'):

            if self.config.hide_ids:
                self.table.del_column('ID')

            if sort:
                # if sort contains the column index instead of the column name, get the column name instead
                if sort.isdecimal():
                    sort = self.table.get_csv_string().split(',')[int(sort) - 1]
            else:
                sort = None

            if limit > 0:
                print(self.table.get_string(sortby=sort, reversesort=reverse, start=0, end=limit))
            else:
                print(self.table.get_string(sortby=sort, reversesort=reverse))

        elif (self.format == 'csv'):
            print(self.table.get_csv_string(delimiter=self.config.delimiter))

    def print(self, contact):
        """Print the data of the specified contact"""

        if (self.format == 'json'):
            print(json.dumps(contact, indent=4))
            return

        id = contact['id']
        name = contact['name']
        email = contact['email'] if 'email' in contact else ''
        phone = contact['phonenumber'] if 'phonenumber' in contact else ''
        method = contact['method'] if 'method' in contact else ''

        self.table.add_row([id, name, email, phone, method])



#!/usr/bin/env python3

import requests
import json
from prettytable import PrettyTable

from .config import Config
from .functions import printError, printWarn

class UserTokens(object):

    def __init__(self, config):
        self.config = config
        self.usertokens = None

        self.table = PrettyTable()
        self.table.field_names = ['Token', 'Name', 'Tags']
        self.table.align['Token'] = 'l'
        self.table.align['Name'] = 'l'
        self.table.align['Tags'] = 'l'

    def fetchData(self):
        """Retrieve the list of all usertokens"""

        # if data is already downloaded, use cached data
        if self.usertokens != None:
            return True

        # check if headers are correctly set for authorization
        if not self.config.headers():
            return False

        if self.config.debug:
            print('GET', self.config.endpoint + 'usertoken?', self.config.params())

        # Make request to API endpoint
        response = requests.get(self.config.endpoint + 'usertoken', params=self.config.params(), headers=self.config.headers())

        # Check status code of response
        if response.status_code == 200:
            # Get list of usertokens from response
            json = response.json()
            if 'tokens' in json:
                self.usertokens = response.json()['tokens']
                return True
            else:
                self.usertokens = None
                return False
        else:
            printError('An error occurred:', response.status_code)
            self.usertokens = None
            return False

    def list(self, token: str = '', format: str = 'table'):
        """Iterate through list of usertokens and print details"""

        if self.fetchData():
            if self.usertokens != None:

                # if JSON was requested and no filters, then just print it without iterating through
                if (format == 'json' and not token):
                    print(json.dumps(self.usertokens, indent=4))
                    return

                for usertoken in self.usertokens:
                    if token:
                        if usertoken['token'] == token:
                            self.print(usertoken)
                            break
                    else:
                        self.print(usertoken)

            if (format == 'table'):
                print(self.table)
            elif (format == 'csv'):
                print(self.table.get_csv_string(delimiter=self.config.delimiter))

    def token(self):
        """Print the data of first usertoken"""

        if self.fetchData() and len(self.usertokens) > 0:
            return self.usertokens[0]['token']

    def create(self, name: str = '', tags: str = ''):
        """Create a new usertoken"""

        # check if headers are correctly set for authorization
        if not self.config.headers():
            return False

        if self.config.debug:
            print('POST', self.config.endpoint + 'usertoken', self.config.params())

        if self.config.readonly:
            return False

        parameters = ''
        if name and tags:
            parameters = 'name=' + name + '&tags=' + tags
        elif name:
            parameters = 'name=' + name

        response = requests.post(self.config.endpoint + 'usertoken', data=parameters, headers=self.config.headers())

        # Check status code of response
        if response.status_code == 200:
            print('Created usertoken')
            return True
        else:
            printError('Failed to create usertoken with response code:', response.status_code)
            return False

    def print(self, usertoken, format: str = 'table'):
        """Print the data of the specified usertoken"""

        if (format == 'json'):
            print(json.dumps(usertoken, indent=4))
            return

        token = usertoken['token']
        name = usertoken['name'] if 'name' in usertoken and usertoken['name'] else ''

        tags = ''
        if 'tags' in usertoken and usertoken['tags']:
            for tag in usertoken['tags']:
                if tags:
                    tags += ', ' + tag
                else:
                    tags = tag

        self.table.add_row([token, name, tags])



#!/usr/bin/env python3

from prettytable import PrettyTable

from .config import Config
from .servers import Servers

class WPToolkit(object):

    def __init__(self, config):
        self.config = config

        self.table = PrettyTable()
        self.table.field_names = ['ID', 'Server name', 'WP sites', 'Alive', 'Outdated', 'Outdated PHP', 'Broken']
        self.table.align['ID'] = 'l'
        self.table.align['Server name'] = 'l'
        self.table.min_width['Server name'] = 24
        self.table.align['WP sites'] = 'r'
        self.table.align['Alive'] = 'r'
        self.table.align['Outdated'] = 'r'
        self.table.align['Outdated PHP'] = 'r'
        self.table.align['Broken'] = 'r'

        self.num_servers_with_wpt = 0
        self.sum_wp_sites_total = 0
        self.sum_wp_sites_alive = 0
        self.sum_wp_sites_outdated = 0
        self.sum_wp_sites_outdated_php = 0
        self.sum_wp_sites_broken = 0

    def printFooter(self, sort: str = '', reverse: bool = False, limit: int = 0):
        """Print table if table format requested"""

        # add summary row as table footer
        self.table.add_row(['', 'Sum of ' + str(self.num_servers_with_wpt) + ' servers', self.sum_wp_sites_total, self.sum_wp_sites_alive, self.sum_wp_sites_outdated, self.sum_wp_sites_outdated_php, self.sum_wp_sites_broken])

        if self.config.hide_ids:
            self.table.del_column('ID')

        # Get string to be printed and create list of elements separated by \n
        list_of_table_lines = self.table.get_string().split('\n')

        # remember summary row
        summary_line = list_of_table_lines[-2]

        # remove summary row again to allow sorting and limiting
        self.table.del_row(len(self.table.rows)-1)

        if sort:
            # if sort contains the column index instead of the column name, get the column name instead
            if sort.isdecimal():
                sort = self.table.get_csv_string().split(',')[int(sort) - 1]
        else:
            sort = None

        if limit > 0:
            list_of_table_lines = self.table.get_string(sortby=sort, reversesort=reverse, start=0, end=limit).split('\n')
        else:
            list_of_table_lines = self.table.get_string(sortby=sort, reversesort=reverse).split('\n')

        # Sorting by multiple columns could be done like this
        # list_of_table_lines = self.table.get_string(sortby=("Col Name 1", "Col Name 2")), reversesort=reverse).split('\n')

        # Print the table
        print('\n'.join(list_of_table_lines))
        print(summary_line)
        print(list_of_table_lines[0])

    def print(self, format: str = 'table', issuesOnly: bool = False, sort: str = '', reverse: bool = False, limit: int = 0):
        """Iterate through all servers and aggregate metrics for those that have WP Toolkit installed"""

        servers = Servers(self.config)
        if servers.fetchData():
            for server in servers.servers:
                id = server['id']
                name = server['name']
                last_data = server['last_data']
                if last_data and 'wp-toolkit' in last_data:
                    wpt_data = last_data['wp-toolkit']
                    if wpt_data:
                        wp_sites_total = wpt_data['WordPress Websites']
                        wp_sites_alive = wpt_data['WordPress Websites - Alive']
                        wp_sites_outdated = wpt_data['WordPress Websites - Outdated']
                        wp_sites_outdated_php = wpt_data['WordPress Websites - Outdated PHP']
                        wp_sites_broken = wpt_data['WordPress Websites - Broken']

                        self.num_servers_with_wpt += 1
                        self.sum_wp_sites_total += wp_sites_total
                        self.sum_wp_sites_alive += wp_sites_alive
                        self.sum_wp_sites_outdated += wp_sites_outdated
                        self.sum_wp_sites_outdated_php += wp_sites_outdated_php
                        self.sum_wp_sites_broken += wp_sites_broken

                        if wp_sites_outdated > 0 or wp_sites_outdated_php > 0 or wp_sites_broken > 0 or not issuesOnly:
                            self.table.add_row([id, name, wp_sites_total, wp_sites_alive, wp_sites_outdated, wp_sites_outdated_php, wp_sites_broken])

        if (format == 'table'):
            self.printFooter(sort=sort, reverse=reverse, limit=limit)
        elif (format == 'csv'):
            print(self.table.get_csv_string(delimiter=self.config.delimiter))



#!/usr/bin/env python3

import requests
import json
from prettytable import PrettyTable

from .config import Config
from .functions import printError, printWarn
from .bcolors import bcolors

class Servers(object):

    def __init__(self, config):
        self.config = config
        self.servers = None
        self.format = 'table'

        self.table = PrettyTable()
        self.table.field_names = ['ID', 'Server name', 'IP Address', 'Status', 'OS', 'CPU Usage %', 'Mem Usage %', 'Disk Usage %', 'Disk Info', 'Tags']
        self.table.align['ID'] = 'l'
        self.table.align['Server name'] = 'l'
        self.table.min_width['Server name'] = 24
        self.table.align['Tags'] = 'l'

        self.sum_cpu_usage = 0
        self.sum_mem_usage = 0
        self.sum_disk_usage = 0
        self.num_servers = 0

    def fetchData(self):
        """Retrieve a list of all monitored servers"""

        # if data is already downloaded, use cached data
        if self.servers != None:
            return True

        # check if headers are correctly set for authorization
        if not self.config.headers():
            return False

        if self.config.debug:
            print('GET', self.config.endpoint + 'servers?', self.config.params())

        # Make request to API endpoint
        response = requests.get(self.config.endpoint + 'servers', params=self.config.params(), headers=self.config.headers())

        # Check status code of response
        if response.status_code == 200:
            # Get list of servers from response
            self.servers = response.json()['servers']
            return True
        else:
            printError('An error occurred:', response.status_code)
            self.servers = None
            return False

    def update(self, serverId: str, tags):
        """Update a specific server and add specified tags to it"""

        data = {
            "tags": tags
        }

        if self.config.debug:
            print('PUT', self.config.endpoint + 'server/' + serverId + '?', data)

        if self.config.readonly:
            return False

        # Make request to API endpoint
        response = requests.put(self.config.endpoint + 'server/' + serverId,  data=json.dumps(data), headers=self.config.headers())

        # Check status code of response
        if response.status_code == 200:
            print('Updated tags of server', serverId, 'to', tags)
            return True
        else:
            printError('Failed to update server', serverId, 'with response code:', response.status_code)
            return False

    def list(self, issuesOnly: bool, sort: str, reverse: bool, limit: int, tags):
        """Iterate through list of server monitors and print details"""

        if self.fetchData():

            # if JSON was requested and no filters, then just print it without iterating through
            if (self.format == 'json' and not (issuesOnly or len(tags) > 0 or limit > 0)):
                print(json.dumps(self.servers, indent=4))
                return

            self.printHeader()

            self.sum_cpu_usage = 0
            self.sum_mem_usage = 0
            self.sum_disk_usage = 0
            self.num_servers = 0

            # Iterate through list of monitors and print urls, etc.
            for server in self.servers:
                if len(tags) == 0:
                    if (not issuesOnly) or self.hasIssue(server):
                        self.print(server)
                elif 'tags' in server:
                    match = True
                    for tag in tags:
                        if not tag in server['tags']:
                            match = False
                            break
                    if match:
                        if (not issuesOnly) or self.hasIssue(server):
                            self.print(server)

            self.printFooter(sort=sort, reverse=reverse, limit=limit)

    def setTags(self, pattern: str, tags):
        """Set the tags for the server specified with pattern. Pattern can be either the server ID or its name"""

        if pattern and len(tags) > 0 and self.fetchData():
            for server in self.servers:
                if pattern == server['id'] or pattern in server['name']:
                    return self.update(server['id'], tags)

        printWarn('No server with given pattern found: ' + pattern)

    def getRecommendation(self, server):
        """Return recommendation text if the specified server has some issue by having a value outside of the expected threshold specified in config file"""

        last_data = server['last_data']

        mem_usage_percent = server['summary']['mem_usage_percent'] if 'summary' in server else 0
        memory_total = last_data['memory']['total'] if 'memory' in last_data else 0
        if memory_total > 0 and mem_usage_percent >= float(self.config.threshold_mem_usage):
            memory_total_gb = memory_total / 1024 / 1024
            memory_total_gb_recommended = round(memory_total_gb * 2)
            if (memory_total_gb_recommended % 2) != 0:
                memory_total_gb_recommended += 1
            return 'Memory is too small for your workload. Please consider upgrading to a larger server with at least ' + str(memory_total_gb_recommended) + ' GB memory.'

        cpu_usage_percent = server['summary']['cpu_usage_percent'] if 'summary' in server else 0
        cores = last_data['cores'] if 'cores' in last_data else 0
        if cores > 0 and cpu_usage_percent >= float(self.config.threshold_cpu_usage):
            cores_recommended = round(cores * 2)
            if (cores % 2) != 0:
                cores += 1
            return 'CPU is too small for your workload. Please consider upgrading to a larger server with at least ' + str(cores_recommended) + ' CPU cores.'

        disk_usage_percent = server['summary']['disk_usage_percent'] if 'summary' in server else 0
        last_data = server['last_data']
        if 'df' in last_data:  # disk_usage_percent >= float(self.config.threshold_disk_usage)
            for disk in last_data['df']:
                mount = disk['mount']
                free_disk_space = disk['free_bytes']
                used_disk_space = disk['used_bytes']
                total_disk_space = free_disk_space + used_disk_space
                free_disk_space_percent = free_disk_space / total_disk_space * 100
                total_disk_space_gb = total_disk_space / 1024 / 1024
                total_disk_space_gb_recommended = round(total_disk_space_gb * 2)
                if (total_disk_space_gb_recommended % 2) != 0:
                    total_disk_space_gb_recommended += 1

                if free_disk_space_percent <= float(self.config.threshold_free_diskspace):
                    return 'Disk volume "' + mount + '" is almost exhausted. Please consider extending your storage volume or upgrading to a larger server with at least ' + str(total_disk_space_gb_recommended) + ' GB disk space for this volume.'

        return ''

    def hasIssue(self, server):
        """Return True if the specified server has some issue by having a value outside of the expected threshold specified in config file"""
        if self.getRecommendation(server):
            return True
        else:
            return False

    def printHeader(self):
        """Print CSV if CSV format requested"""
        if (self.format == 'csv'):
            print(self.config.delimiter.join(self.table.field_names))

    def printFooter(self, sort: str = '', reverse: bool = False, limit: int = 0):
        """Print table if table format requested"""

        if (self.format == 'table'):

            avg_cpu_usage = self.sum_cpu_usage / self.num_servers if self.sum_cpu_usage > 0 and self.num_servers > 0 else 0
            avg_mem_usage = self.sum_mem_usage / self.num_servers if self.sum_mem_usage > 0 and self.num_servers > 0 else 0
            avg_disk_usage = self.sum_disk_usage / self.num_servers if self.sum_disk_usage > 0 and self.num_servers > 0 else 0

            if avg_cpu_usage >= float(self.config.threshold_cpu_usage):
                avg_cpu_usage_text = f"{bcolors.FAIL}" + "{:.1f}".format(avg_cpu_usage) + '%' + f"{bcolors.ENDC}"
            else:
               avg_cpu_usage_text = "{:.1f}".format(avg_cpu_usage) + '%'

            if avg_mem_usage >= float(self.config.threshold_mem_usage):
                avg_mem_usage_text = f"{bcolors.FAIL}" + "{:.1f}".format(avg_mem_usage) + '%' + f"{bcolors.ENDC}"
            else:
               avg_mem_usage_text = "{:.1f}".format(avg_mem_usage) + '%'

            if avg_disk_usage >= float(self.config.threshold_disk_usage):
                avg_disk_usage_text = f"{bcolors.FAIL}" + "{:.1f}".format(avg_disk_usage) + '%' + f"{bcolors.ENDC}"
            else:
               avg_disk_usage_text = "{:.1f}".format(avg_disk_usage) + '%'

            # add average row as table footer
            self.table.add_row(['', 'Average of ' + str(self.num_servers) + ' servers', '', '', '', avg_cpu_usage_text, avg_mem_usage_text, avg_disk_usage_text, '', ''])

            if self.config.hide_ids:
                self.table.del_column('ID')

            # Get string to be printed and create list of elements separated by \n
            list_of_table_lines = self.table.get_string().split('\n')

            # remember summary row
            summary_line = list_of_table_lines[-2]

            # remove summary row again to allow sorting and limiting
            self.table.del_row(len(self.table.rows)-1)

            if sort:
                # if sort contains the column index instead of the column name, get the column name instead
                if sort.isdecimal():
                    sort = self.table.get_csv_string().split(',')[int(sort) - 1]
            else:
                sort = None

            if limit > 0:
                list_of_table_lines = self.table.get_string(sortby=sort, reversesort=reverse, start=0, end=limit).split('\n')
            else:
                list_of_table_lines = self.table.get_string(sortby=sort, reversesort=reverse).split('\n')

            # Sorting by multiple columns could be done like this
            # list_of_table_lines = self.table.get_string(sortby=("Col Name 1", "Col Name 2")), reversesort=reverse).split('\n')

            # Print the table
            print('\n'.join(list_of_table_lines))
            print(summary_line)
            print(list_of_table_lines[0])

    def print(self, server):
        """Print the data of the specified server monitor"""

        if (self.format == 'json'):
            print(json.dumps(server, indent=4))
            return

        id = server['id']
        name = server['name']
        os = server['os'] if 'os' in server else ''
        agent_version = server['agent_version'] if 'agent_version' in server else ''
        status = server['status'] if 'status' in server else ''
        last_data = server['last_data']
        uptime_seconds = last_data['uptime']['seconds'] if 'uptime' in last_data else 0
        cores = last_data['cores'] if 'cores' in last_data else 0
        memory_used = last_data['memory']['used'] if 'memory' in last_data else 0
        memory_free = last_data['memory']['free'] if 'memory' in last_data else 0
        memory_available = last_data['memory']['available'] if 'memory' in last_data else 0
        memory_total = last_data['memory']['total'] if 'memory' in last_data else 0
        connecting_ip = server['connecting_ip'] if 'connecting_ip' in server else ''
        if 'ip_whois' in server and server['ip_whois']:
            ip_whois = server['ip_whois']
            ip_address = ip_whois['ip'] if 'ip' in ip_whois else ''
            ip_country = ip_whois['country'] if 'country' in ip_whois else ''
            ip_hoster = ip_whois['org'] if 'org' in ip_whois else ''
        else:
            ip_address = ''
            ip_country = ''
            ip_hoster = ''
        cpu_usage_percent = server['summary']['cpu_usage_percent'] if 'summary' in server else 0
        mem_usage_percent = server['summary']['mem_usage_percent'] if 'summary' in server else 0
        disk_usage_percent = server['summary']['disk_usage_percent'] if 'summary' in server else 0

        self.sum_cpu_usage = self.sum_cpu_usage + cpu_usage_percent
        self.sum_mem_usage = self.sum_mem_usage + mem_usage_percent
        self.sum_disk_usage = self.sum_disk_usage + disk_usage_percent
        self.num_servers = self.num_servers + 1

        if cpu_usage_percent >= float(self.config.threshold_cpu_usage):
            cpu_usage_percent_text = f"{bcolors.FAIL}" + "{:.1f}".format(cpu_usage_percent) + '%' + f"{bcolors.ENDC}"
        else:
            cpu_usage_percent_text = "{:.1f}".format(cpu_usage_percent) + '%'

        if mem_usage_percent >= float(self.config.threshold_mem_usage):
            mem_usage_percent_text = f"{bcolors.FAIL}" + "{:.1f}".format(mem_usage_percent) + '%' + f"{bcolors.ENDC}"
        else:
            mem_usage_percent_text = "{:.1f}".format(mem_usage_percent) + '%'

        if disk_usage_percent >= float(self.config.threshold_disk_usage):
            disk_usage_percent_text = f"{bcolors.FAIL}" + "{:.1f}".format(disk_usage_percent) + '%' + f"{bcolors.ENDC}"
        else:
            disk_usage_percent_text = "{:.1f}".format(disk_usage_percent) + '%'

        tags = ''
        if 'tags' in server and server['tags']:
            for tag in server['tags']:
                if tags:
                    tags += ', ' + tag
                else:
                    tags = tag

        disk_info = ''
        if 'df' in last_data:
            for disk in last_data['df']:
                free_disk_space = disk['free_bytes']
                used_disk_space = disk['used_bytes']
                total_disk_space = free_disk_space + used_disk_space
                free_disk_space_percent = free_disk_space / total_disk_space * 100
                mount = disk['mount']

                # add separator
                if disk_info:
                    disk_info += ', '

                if free_disk_space_percent <= float(self.config.threshold_free_diskspace):
                    disk_info += f"{bcolors.FAIL}" + "{:.0f}".format(free_disk_space_percent) + "% free on " + mount + f"{bcolors.ENDC}"
                else:
                    disk_info += "{:.0f}".format(free_disk_space_percent) + "% free on " + mount

        if (self.format == 'csv'):
            print(self.config.delimiter.join([id, name, ip_address, status, os, str(cpu_usage_percent) + '%', str(mem_usage_percent) + '%', str(disk_usage_percent) + '%', disk_info, tags]))
        else:
            self.table.add_row([id, name, ip_address, status, os, cpu_usage_percent_text, mem_usage_percent_text, disk_usage_percent_text, disk_info, tags])



#!/usr/bin/env python3

from prettytable import PrettyTable

from .config import Config
from .servers import Servers
from .sites import Sites
from .contacts import Contacts
from .usertokens import UserTokens
from .functions import printError, printWarn
from .bcolors import bcolors

class Statistics(object):

    def __init__(self, config):
        self.config = config

        self.table = PrettyTable()
        self.table.field_names = ['Value', 'Metric']
        self.table.align['Value'] = 'r'
        self.table.align['Metric'] = 'l'

    def print(self, format: str = 'table'):
        """Iterate through all assets and print statistics"""

        servers = Servers(self.config)
        if servers.fetchData():
            sum_cpu_usage = 0
            sum_mem_usage = 0
            sum_disk_usage = 0
            num_servers = len(servers.servers)
            self.table.add_row([len(servers.servers), 'Servers'])

            for server in servers.servers:
                sum_cpu_usage = sum_cpu_usage + server['summary']['cpu_usage_percent'] if 'summary' in server else 0
                sum_mem_usage = sum_mem_usage + server['summary']['mem_usage_percent'] if 'summary' in server else 0
                sum_disk_usage = sum_disk_usage + server['summary']['disk_usage_percent'] if 'summary' in server else 0

            avg_cpu_usage = sum_cpu_usage / num_servers if sum_cpu_usage > 0 and num_servers > 0 else 0
            avg_mem_usage = sum_mem_usage / num_servers if sum_mem_usage > 0 and num_servers > 0 else 0
            avg_disk_usage = sum_disk_usage / num_servers if sum_disk_usage > 0 and num_servers > 0 else 0

            if avg_cpu_usage >= float(self.config.threshold_cpu_usage):
                avg_cpu_usage_text = f"{bcolors.FAIL}" + "{:.1f}".format(avg_cpu_usage) + f"{bcolors.ENDC}"
            else:
               avg_cpu_usage_text = "{:.1f}".format(avg_cpu_usage)

            if avg_mem_usage >= float(self.config.threshold_mem_usage):
                avg_mem_usage_text = f"{bcolors.FAIL}" + "{:.1f}".format(avg_mem_usage) + f"{bcolors.ENDC}"
            else:
               avg_mem_usage_text = "{:.1f}".format(avg_mem_usage)

            if avg_disk_usage >= float(self.config.threshold_disk_usage):
                avg_disk_usage_text = f"{bcolors.FAIL}" + "{:.1f}".format(avg_disk_usage) + f"{bcolors.ENDC}"
            else:
               avg_disk_usage_text = "{:.1f}".format(avg_disk_usage)

            self.table.add_row([avg_cpu_usage_text, '% avg cpu usage of all ' + str(num_servers) + ' servers'])
            self.table.add_row([avg_mem_usage_text, '% avg mem usage of all ' + str(num_servers) + ' servers'])
            self.table.add_row([avg_disk_usage_text, '% avg disk usage of all ' + str(num_servers) + ' servers'])

        sites = Sites(self.config)
        if sites.fetchData():
            sum_uptime = 0
            sum_ttfb = 0
            num_monitors = len(sites.monitors)
            self.table.add_row([len(sites.monitors), 'Sites'])

            for monitor in sites.monitors:
                uptime_percentage = float(monitor['uptime_percentage'])

                if 'last_check' in monitor and 'ttfb' in monitor['last_check']:
                    ttfb = float(monitor['last_check']['ttfb'])
                    sum_uptime = sum_uptime + uptime_percentage
                    sum_ttfb = sum_ttfb + ttfb

            avg_uptime = sum_uptime / num_monitors if sum_uptime > 0 and num_monitors > 0 else 0
            avg_ttfb = sum_ttfb / num_monitors if sum_ttfb > 0 and num_monitors > 0 else 0

            if avg_uptime <= float(self.config.threshold_uptime):
                uptime_percentage_text = f"{bcolors.FAIL}" + "{:.4f}".format(avg_uptime) + f"{bcolors.ENDC}"
            else:
                uptime_percentage_text = "{:.4f}".format(avg_uptime)

            if avg_ttfb >= float(self.config.threshold_ttfb):
                ttfb_text = f"{bcolors.FAIL}" + "{:.2f}".format(avg_ttfb) + f"{bcolors.ENDC}"
            else:
                ttfb_text = "{:.2f}".format(avg_ttfb)

            self.table.add_row([uptime_percentage_text, '% avg uptime of all ' + str(num_monitors) + ' sites'])
            self.table.add_row([ttfb_text, 'sec avg ttfb of all ' + str(num_monitors) + ' sites'])

        contacts = Contacts(self.config)
        if contacts.fetchData():
            self.table.add_row([len(contacts.contacts), 'Contacts'])

        usertokens = UserTokens(self.config)
        if usertokens.fetchData():
            self.table.add_row([len(usertokens.usertokens), 'User Tokens'])

        if (format == 'table'):
            print(self.table)
        elif (format == 'csv'):
            print(self.table.get_csv_string(delimiter=self.config.delimiter))



#!/usr/bin/env python3

import os
import argparse
import json
import webbrowser

# suprisingly this works in PyPi, but not locally. For local usage replace ".lib." with "lib."
# use "pip install -e ." to use "360monitoring" command with latest dev build locally based on local code.
from .lib.config import Config
from .lib.contacts import Contacts
from .lib.recommendations import Recommendations
from .lib.servers import Servers
from .lib.sites import Sites
from .lib.statistics import Statistics
from .lib.usertokens import UserTokens
from .lib.wptoolkit import WPToolkit

__version__ = '1.0.13'

cfg = Config(__version__)
cli = argparse.ArgumentParser(prog='360monitoring', description='CLI for 360 Monitoring')
cli_subcommands = dict()

def check_columns(columns):
    """Show or hide columns in ASCII table view"""
    for column in columns:
        if '0id' == column.lower():
            cfg.hide_ids = True
        elif 'id' == column.lower():
            cfg.hide_ids = False

# --- config functions ---

def config_print(args):
    """Sub command for config print"""
    cfg.print()

def config_save(args):
    """Sub command for config save"""
    if args.api_key:
        cfg.api_key = args.api_key

    cfg.saveToFile()

def config(args):
    """Sub command for config"""
    config_print(args)

# --- contacts functions ---

def contacts_add(args):
    """Sub command for contacts add"""
    contacts = Contacts(cfg)

    if args.file:
        if os.path.isfile(args.file):
            with open(args.file) as file:
                lines = file.readlines()
                for line in lines:
                    contacts.add(line.strip())
        else:
            print('ERROR: File', args.file, 'to import not found')
    elif args.name:
        contacts.add(name=args.name, email=args.email, sms=args.sms)
    else:
        print('You need to specify at least a name with --name [name]')

def contacts_list(args):
    """Sub command for contacts list"""
    check_columns(args.columns)
    contacts = Contacts(cfg)
    contacts.format = args.output
    contacts.list(id=args.id, name=args.name, email=args.email, phone=args.phone, sort=args.sort, reverse=args.reverse, limit=args.limit)

def contacts_remove(args):
    """Sub command for contacts remove"""
    contacts = Contacts(cfg)
    contacts.remove(id=args.id, name=args.name, email=args.email, phone=args.phone)

def contacts(args):
    """Sub command for contacts"""
    cli_subcommands[args.subparser].print_help()

# --- dashboard functions ---

def dashboard(args):
    """Sub command for dashboard"""
    webbrowser.open('https://monitoring.platform360.io/')

# --- recommendations functions ---

def recommendations(args):
    """Sub command for recommendations"""
    recommendations = Recommendations(cfg)
    recommendations.print(format=args.output)

# --- servers functions ---

def servers_add(args):
    """Sub command for servers add"""
    usertokens = UserTokens(cfg)
    token = usertokens.token()
    if not token:
        print('First create a user token by executing:')
        print()
        print('360monitoring usertokens create')
        print('360monitoring usertokens list')
        print()
        token = '[YOUR_USER_TOKEN]'

    print('Please login via SSH to each of the servers you would like to add and execute the following command:')
    print()
    print('wget -q -N monitoring.platform360.io/agent360.sh && bash agent360.sh', token)

def servers_list(args):
    """Sub command for servers list"""
    check_columns(args.columns)
    servers = Servers(cfg)
    servers.format = args.output
    servers.list(args.issues, args.sort, args.reverse, args.limit, args.tag)

def servers_remove(args):
    """Sub command for servers remove"""
    print("Please login via SSH to each of the servers you would like to remove.")
    print("First stop the monitoring agent by running \"service agent360 stop\" then run \"pip3 uninstall agent360\". After 15 minutes you are able to remove the server.")

def servers_update(args):
    """Sub command for servers update"""
    servers = Servers(cfg)
    pattern = ''
    if args.id:
        pattern = args.id
    elif args.name:
        pattern = args.name
    servers.setTags(pattern, args.tag)

def servers(args):
    """Sub command for servers"""
    cli_subcommands[args.subparser].print_help()

# --- signup functions ---

def signup(args):
    """Sub command for signup"""
    webbrowser.open('https://360monitoring.com/monitoring-trial/')

# --- sites functions ---

def sites_add(args):
    """Sub command for sites add"""
    sites = Sites(cfg)

    if args.file:
        if os.path.isfile(args.file):
            with open(args.file) as file:
                lines = file.readlines()
                for line in lines:
                    sites.add(line.strip())
        else:
            print('ERROR: File', args.file, 'to import not found')
    elif args.url:
        sites.add(args.url, protocol=args.protocol, name=args.name, force=args.force)
    else:
        print('You need to specify at least a name with --name [name]')

def sites_list(args):
    """Sub command for sites list"""
    check_columns(args.columns)
    sites = Sites(cfg)
    sites.format = args.output
    sites.list(id=args.id, url=args.url, name=args.name, location=args.location, pattern=args.pattern, issuesOnly=args.issues, sort=args.sort, reverse=args.reverse, limit=args.limit)

def sites_remove(args):
    """Sub command for sites remove"""
    sites = Sites(cfg)
    sites.remove(id=args.id, url=args.url, name=args.name, location=args.location, pattern=args.pattern)

def sites(args):
    """Sub command for sites"""
    cli_subcommands[args.subparser].print_help()

# --- statistics functions ---

def statistics(args):
    """Sub command for statistics"""
    statistics = Statistics(cfg)
    statistics.print(format=args.output)

# --- usertokens functions ---

def usertokens_create(args):
    """Sub command for usertokens create"""
    usertokens = UserTokens(cfg)
    usertokens.create(name=args.name, tags=args.tag)

def usertokens_list(args):
    """Sub command for usertokens list"""
    usertokens = UserTokens(cfg)
    usertokens.list(format=args.output)

def usertokens(args):
    """Sub command for usertokens"""
    cli_subcommands[args.subparser].print_help()

# --- wptoolkit functions ---

def wptoolkit(args):
    """Sub command for wptoolkit"""
    check_columns(args.columns)
    wptoolkit = WPToolkit(cfg)
    wptoolkit.print(format=args.output, issuesOnly=args.issues, sort=args.sort, reverse=args.reverse, limit=args.limit)

# --- configure & parse CLI ---

def performCLI():
    """Parse the command line parameters and call the related functions"""

    subparsers = cli.add_subparsers(title='commands', dest='subparser')
    cli.add_argument('-v', '--version', action='store_true', help='print CLI version')

    # config

    cli_config = subparsers.add_parser('config', help='configure connection to 360 Monitoring account')
    cli_config.set_defaults(func=config)

    config_subparsers = cli_config.add_subparsers(title='commands', dest='subparser')

    cli_config_print = config_subparsers.add_parser('print', help='print current settings for 360 Monitoring')
    cli_config_print.set_defaults(func=config_print)

    cli_config_save = config_subparsers.add_parser('save', help='save current settings for 360 Monitoring to ' + cfg.filename)
    cli_config_save.set_defaults(func=config_save)
    cli_config_save.add_argument('-a', '--api-key', metavar='key', help='specify your API KEY for 360 Monitoring')

    # contacts

    cli_contacts = subparsers.add_parser('contacts', help='list and manage contacts')
    cli_contacts.set_defaults(func=contacts)
    cli_contacts_subparsers = cli_contacts.add_subparsers(title='commands', dest='subparser')

    cli_contacts_add = cli_contacts_subparsers.add_parser('add', help='add a new contact')
    cli_contacts_add.set_defaults(func=contacts_add)
    cli_contacts_add.add_argument('--name', nargs='?', metavar='name', help='name of the new contact')
    cli_contacts_add.add_argument('--email', nargs='?', default='', metavar='email', help='email address of the new contact')
    cli_contacts_add.add_argument('--sms', nargs='?', default='', metavar='phone', help='mobile phone number of the new contact (optional)')
    cli_contacts_add.add_argument('--file', nargs='?', default='', metavar='file', help='file containing one contact per line to add')

    cli_contacts_list = cli_contacts_subparsers.add_parser('list', help='list contacts')
    cli_contacts_list.set_defaults(func=contacts_list)
    cli_contacts_list.add_argument('--id', nargs='?', default='', metavar='id', help='list contact with given ID')
    cli_contacts_list.add_argument('--name', nargs='?', default='', metavar='name', help='list contact with given name')
    cli_contacts_list.add_argument('--email', nargs='?', default='', metavar='email', help='list contact with given email address')
    cli_contacts_list.add_argument('--phone', nargs='?', default='', metavar='phone', help='list contact with given phone number')

    cli_contacts_list.add_argument('--columns', nargs='*', default='', metavar='col', help='specify columns to print in table view or remove columns with 0 as prefix e.g. "0id"')
    cli_contacts_list.add_argument('--sort', nargs='?', default='', metavar='col', help='sort by specified column. Reverse sort by adding --reverse')
    cli_contacts_list.add_argument('--reverse', action='store_true', help='show in descending order. Works only together with --sort')
    cli_contacts_list.add_argument('--limit', nargs='?', default=0, type=int, metavar='n', help='limit the number of printed items')

    cli_contacts_list.add_argument('--output', choices=['json', 'csv', 'table'], default='table', help='output format for the data')
    cli_contacts_list.add_argument('--json', action='store_const', const='json', dest='output', help='print data in JSON format')
    cli_contacts_list.add_argument('--csv', action='store_const', const='csv', dest='output', help='print data in CSV format')
    cli_contacts_list.add_argument('--table', action='store_const', const='table', dest='output', help='print data as ASCII table')

    cli_contacts_remove = cli_contacts_subparsers.add_parser('remove', help='remove a contact')
    cli_contacts_remove.set_defaults(func=contacts_remove)
    cli_contacts_remove.add_argument('--id', nargs='?', default='', metavar='id', help='remove contact with given ID')
    cli_contacts_remove.add_argument('--name', nargs='?', default='', metavar='name', help='remove contact with given name')
    cli_contacts_remove.add_argument('--email', nargs='?', default='', metavar='email', help='remove contact with given email address')
    cli_contacts_remove.add_argument('--phone', nargs='?', default='', metavar='phone', help='remove contact with given phone number')

    # dashboard

    cli_dashboard = subparsers.add_parser('dashboard', help='open 360 Monitoring Dashboard in your Web Browser')
    cli_dashboard.set_defaults(func=dashboard)

    # recommendations

    cli_recommendations = subparsers.add_parser('recommendations', help='show upgrade recommendations for servers that exceed their limits')
    cli_recommendations.set_defaults(func=recommendations)
    cli_recommendations.add_argument('--output', choices=['csv', 'table'], default='table', help='output format for the data')
    cli_recommendations.add_argument('--csv', action='store_const', const='csv', dest='output', help='print data in CSV format')
    cli_recommendations.add_argument('--table', action='store_const', const='table', dest='output', help='print data as ASCII table')

    # servers

    cli_servers = subparsers.add_parser('servers', help='list and manage monitored servers')
    cli_servers.set_defaults(func=servers)
    cli_servers_subparsers = cli_servers.add_subparsers(title='commands', dest='subparser')

    cli_servers_add = cli_servers_subparsers.add_parser('add', help='activate monitoring for a server')
    cli_servers_add.set_defaults(func=servers_add)

    cli_servers_list = cli_servers_subparsers.add_parser('list', help='list monitored servers')
    cli_servers_list.set_defaults(func=servers_list)
    cli_servers_list.add_argument('--id', nargs='?', default='', metavar='id', help='update server with given ID')
    cli_servers_list.add_argument('--name', nargs='?', default='', metavar='name', help='update server with given name')
    cli_servers_list.add_argument('--tag', nargs='*', default='', metavar='tag', help='only list servers matching these tags')
    cli_servers_list.add_argument('--issues', action='store_true', help='show only servers with issues')

    cli_servers_list.add_argument('--columns', nargs='*', default='', metavar='col', help='specify columns to print in table view or remove columns with 0 as prefix e.g. "0id"')
    cli_servers_list.add_argument('--sort', nargs='?', default='', metavar='col', help='sort by specified column. Reverse sort by adding --reverse')
    cli_servers_list.add_argument('--reverse', action='store_true', help='show in descending order. Works only together with --sort')
    cli_servers_list.add_argument('--limit', nargs='?', default=0, type=int, metavar='n', help='limit the number of printed items')

    cli_servers_list.add_argument('--output', choices=['json', 'csv', 'table'], default='table', help='output format for the data')
    cli_servers_list.add_argument('--json', action='store_const', const='json', dest='output', help='print data in JSON format')
    cli_servers_list.add_argument('--csv', action='store_const', const='csv', dest='output', help='print data in CSV format')
    cli_servers_list.add_argument('--table', action='store_const', const='table', dest='output', help='print data as ASCII table')

    cli_servers_remove = cli_servers_subparsers.add_parser('remove', help='remove monitoring for a server')
    cli_servers_remove.set_defaults(func=servers_remove)

    cli_servers_update = cli_servers_subparsers.add_parser('update', help='set tags for a server')
    cli_servers_update.set_defaults(func=servers_update)
    cli_servers_update.add_argument('--id', nargs='?', default='', metavar='id', help='update server with given ID')
    cli_servers_update.add_argument('--name', nargs='?', default='', metavar='name', help='update server with given name')
    cli_servers_update.add_argument('--tag', nargs='*', default='', metavar='tag', help='set these tags for one or more servers specified')

    # signup

    cli_signup = subparsers.add_parser('signup', help='sign up for 360 Monitoring')
    cli_signup.set_defaults(func=signup)

    # sites

    cli_sites = subparsers.add_parser('sites', help='list and manage monitored websites')
    cli_sites.set_defaults(func=sites)
    cli_sites_subparsers = cli_sites.add_subparsers(title='commands', dest='subparser')

    cli_sites_add = cli_sites_subparsers.add_parser('add', help='activate monitoring for a site')
    cli_sites_add.set_defaults(func=sites_add)
    cli_sites_add.add_argument('--url', nargs='?', metavar='url', help='url of site that should be monitored')
    cli_sites_add.add_argument('--name', nargs='?', metavar='name', help='name of site that should be monitored (optional)')
    cli_sites_add.add_argument('--protocol', nargs='?', default='https', metavar='protocol', help='specify a different protocol than https')
#    cli_sites_add.add_argument('--port', nargs='?', default=443, type=int, metavar='port', help='specify a different port than 443')
    cli_sites_add.add_argument('--force', action='store_true', help='add new monitor even if already exists')
    cli_sites_add.add_argument('--file', nargs='?', default='', metavar='file', help='file containing one URL per line to monitor')

    cli_sites_list = cli_sites_subparsers.add_parser('list', help='list sites')
    cli_sites_list.set_defaults(func=sites_list)
    cli_sites_list.add_argument('--id', nargs='?', default='', metavar='id', help='list site with given ID')
    cli_sites_list.add_argument('--url', nargs='?', default='', metavar='url', help='list site with given url')
    cli_sites_list.add_argument('--name', nargs='?', default='', metavar='name', help='list site with given name')
    cli_sites_list.add_argument('--location', nargs='?', default='', metavar='location', help='list sites monitored from given location')
    cli_sites_list.add_argument('--pattern', nargs='?', default='', metavar='pattern', help='list sites with pattern included in URL')
    cli_sites_list.add_argument('--issues', action='store_true', help='show only sites with issues')

    cli_sites_list.add_argument('--columns', nargs='*', default='', metavar='col', help='specify columns to print in table view or remove columns with 0 as prefix e.g. "0id"')
    cli_sites_list.add_argument('--sort', nargs='?', default='', metavar='col', help='sort by specified column. Reverse sort by adding --reverse')
    cli_sites_list.add_argument('--reverse', action='store_true', help='show in descending order. Works only together with --sort')
    cli_sites_list.add_argument('--limit', nargs='?', default=0, type=int, metavar='n', help='limit the number of printed items')

    cli_sites_list.add_argument('--output', choices=['json', 'csv', 'table'], default='table', help='output format for the data')
    cli_sites_list.add_argument('--json', action='store_const', const='json', dest='output', help='print data in JSON format')
    cli_sites_list.add_argument('--csv', action='store_const', const='csv', dest='output', help='print data in CSV format')
    cli_sites_list.add_argument('--table', action='store_const', const='table', dest='output', help='print data as ASCII table')

    cli_sites_remove = cli_sites_subparsers.add_parser('remove', help='remove a contact')
    cli_sites_remove.set_defaults(func=sites_remove)
    cli_sites_remove.add_argument('--id', nargs='?', default='', metavar='id', help='remove site with given ID')
    cli_sites_remove.add_argument('--url', nargs='?', default='', metavar='url', help='remove site with given url')
    cli_sites_remove.add_argument('--name', nargs='?', default='', metavar='name', help='remove site with given name')
    cli_sites_remove.add_argument('--location', nargs='?', default='', metavar='location', help='remove sites monitored from given location')
    cli_sites_remove.add_argument('--pattern', nargs='?', default='', metavar='pattern', help='remove sites with pattern included in URL')

    # statistics

    cli_statistics = subparsers.add_parser('statistics', help='print statistics')
    cli_statistics.set_defaults(func=statistics)
    cli_statistics.add_argument('--output', choices=['csv', 'table'], default='table', help='output format for the data')
    cli_statistics.add_argument('--csv', action='store_const', const='csv', dest='output', help='print data in CSV format')
    cli_statistics.add_argument('--table', action='store_const', const='table', dest='output', help='print data as ASCII table')

    # user tokens

    cli_usertokens = subparsers.add_parser('usertokens', help='list or create usertokens')
    cli_usertokens.set_defaults(func=usertokens)
    cli_usertokens_subparsers = cli_usertokens.add_subparsers(title='commands', dest='subparser')

    cli_usertokens_create = cli_usertokens_subparsers.add_parser('create', help='create new user token')
    cli_usertokens_create.set_defaults(func=usertokens_create)
    cli_usertokens_create.add_argument('--name', nargs='?', default='', metavar='name', help='name of the new token (optional)')
    cli_usertokens_create.add_argument('--tag', nargs='?', default='', metavar='tag', help='set a tag for the new token (optional)')

    cli_usertokens_list = cli_usertokens_subparsers.add_parser('list', help='list usertokens')
    cli_usertokens_list.set_defaults(func=usertokens_list)
    cli_usertokens_list.add_argument('--output', choices=['json', 'csv', 'table'], default='table', help='output format for the data')
    cli_usertokens_list.add_argument('--json', action='store_const', const='json', dest='output', help='print data in JSON format')
    cli_usertokens_list.add_argument('--csv', action='store_const', const='csv', dest='output', help='print data in CSV format')
    cli_usertokens_list.add_argument('--table', action='store_const', const='table', dest='output', help='print data as ASCII table')

    # wptoolkit

    cli_wptoolkit = subparsers.add_parser('wptoolkit', help='list statistics of WP Toolkit if installed')
    cli_wptoolkit.set_defaults(func=wptoolkit)

    cli_wptoolkit.add_argument('--issues', action='store_true', help='show only servers with WordPress issues')
    cli_wptoolkit.add_argument('--columns', nargs='*', default='', metavar='col', help='specify columns to print in table view or remove columns with 0 as prefix e.g. "0id"')
    cli_wptoolkit.add_argument('--sort', nargs='?', default='', metavar='col', help='sort by specified column. Reverse sort by adding --reverse')
    cli_wptoolkit.add_argument('--reverse', action='store_true', help='show in descending order. Works only together with --sort')
    cli_wptoolkit.add_argument('--limit', nargs='?', default=0, type=int, metavar='n', help='limit the number of printed items')

    cli_wptoolkit.add_argument('--output', choices=['csv', 'table'], default='table', help='output format for the data')
    cli_wptoolkit.add_argument('--csv', action='store_const', const='csv', dest='output', help='print data in CSV format')
    cli_wptoolkit.add_argument('--table', action='store_const', const='table', dest='output', help='print data as ASCII table')

    # Parse

    cli_subcommands['config'] = cli_config
    cli_subcommands['contacts'] = cli_contacts
    cli_subcommands['dashboard'] = cli_dashboard
    cli_subcommands['recommendations'] = cli_recommendations
    cli_subcommands['servers'] = cli_servers
    cli_subcommands['signup'] = cli_signup
    cli_subcommands['sites'] = cli_sites
    cli_subcommands['statistics'] = cli_statistics
    cli_subcommands['usertokens'] = cli_usertokens
    cli_subcommands['wptoolkit'] = cli_wptoolkit

    args = cli.parse_args()
    if args.subparser == None:
        if args.version:
            print('360 Monitoring CLI Version:', __version__)
        elif 'func' in args:
            # recommendations, statistics, signup, wptoolkit and dashboard is shown directly without subparser
            if args.func == config:
                cli_config.print_help()
            elif args.func == contacts:
                cli_contacts.print_help()
            elif args.func == servers:
                cli_servers.print_help()
            elif args.func == sites:
                cli_sites.print_help()
            elif args.func == usertokens:
                cli_usertokens.print_help()
            else:
                cli.print_help()
        else:
            cli.print_help()
    else:
        args.func(args)

def main():
    performCLI()

if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import requests
import json
from prettytable import PrettyTable

from .config import Config
from .functions import printError, printWarn
from .bcolors import bcolors

class Sites(object):

    def __init__(self, config):
        self.config = config
        self.monitors = None
        self.format = 'table'

        self.table = PrettyTable()
        self.table.field_names = ['ID', 'URL', 'Status', 'Uptime %', 'Time to first Byte', 'Location']
        self.table.align['ID'] = 'l'
        self.table.align['URL'] = 'l'
        self.table.min_width['URL'] = 25
        self.table.align['Status'] = 'l'
        self.table.align['Uptime %'] = 'r'
        self.table.align['Time to first Byte'] = 'c'
        self.table.align['Location'] = 'c'

        self.sum_uptime = 0
        self.sum_ttfb = 0
        self.num_monitors = 0

    def fetchData(self):
        """Retrieve the list of all website monitors"""

        # if data is already downloaded, use cached data
        if self.monitors != None:
            return True

        # check if headers are correctly set for authorization
        if not self.config.headers():
            return False

        if self.config.debug:
            print('GET', self.config.endpoint + 'monitors?', self.config.params())

        # Make request to API endpoint
        response = requests.get(self.config.endpoint + 'monitors', params=self.config.params(), headers=self.config.headers())

        # Check status code of response
        if response.status_code == 200:
            # Get list of servers from response
            self.monitors = response.json()['monitors']
            return True
        else:
            printError('An error occurred:', response.status_code)
            self.monitors = None
            return False

    def list(self, id: str = '', url: str = '', name: str = '', location: str = '', pattern: str = '', issuesOnly: bool = False, sort: str = '', reverse: bool = False, limit: int = 0):
        """Iterate through list of web monitors and print details"""

        if self.fetchData():

            # if JSON was requested and no filters, then just print it without iterating through
            if (self.format == 'json' and not (id or url or name or location or pattern or issuesOnly or limit > 0)):
                print(json.dumps(self.monitors, indent=4))
                return

            self.printHeader()

            self.sum_uptime = 0
            self.sum_ttfb = 0
            self.num_monitors = 0

            for monitor in self.monitors:
                if (id or url or name or location or pattern):
                    if (id and monitor['id'] == id) \
                        or (url and monitor['url'] == url) \
                        or (name and 'name' in monitor and monitor['name'] == name) \
                        or (location and location in monitor['monitor']['name']) \
                        or (pattern and pattern in monitor['url']):
                        if (not issuesOnly) or self.hasIssue(monitor):
                            self.print(monitor)
                else:
                    if (not issuesOnly) or self.hasIssue(monitor):
                        self.print(monitor)

            self.printFooter(sort=sort, reverse=reverse, limit=limit)

    def add(self, url: str, protocol: str = 'https', name: str = '', force: bool = False):
        """Add a monitor for the given URL"""

        if url and self.fetchData():

            # urls do not include the protocol
            url = url.replace('https://', '').replace('http://', '')

            # use the url as name if not specified otherwise
            if not name:
                name = url

            # use https as protocol if not specified otherwise
            if not protocol:
                protocol = 'https'

            # check if monitored url already exists unless --force is specified
            if not force:
                for monitor in self.monitors:
                    if monitor['url'] == url:
                        print(url, 'already exists and will not be added')
                        return

            # other parameters:
            #   port: int (e.g. 443, 80)
            #   keyword: string that needs to be in the http response body (e.g. "error")
            #   redirects: int (e.g. 3 max redirects; 0 for no redirects)
            #   timeout: int (e.g. 30 seconds)

            # Make request to API endpoint
            data = {
                'url': url,
                'name': name,
                'protocol': protocol
            }

            if self.config.debug:
                print('POST', self.config.endpoint + 'monitors?', data)

            if self.config.readonly:
                return False

            response = requests.post(self.config.endpoint + 'monitors',  data=json.dumps(data), headers=self.config.headers())

            # Check status code of response
            if response.status_code == 200:
                print('Added site monitor:', url)
                return True
            else:
                printError('Failed to add site monitor', url, 'with response code:', response.status_code)
                return False

        else:
            return False

    def remove(self, id: str = '', url: str = '', name: str = '', location: str = '', pattern: str = ''):
        """Remove the monitor for the given URL"""

        removed = 0
        if (id or url or name or location or pattern) and self.fetchData():
            for monitor in self.monitors:
                curr_id = monitor['id']
                curr_url = monitor['url']
                curr_name = monitor['name'] if 'name' in monitor else ''
                curr_location = monitor['monitor']['name']

                if (id == curr_id) \
                    or (url and url == curr_url) \
                    or (name and name == curr_name) \
                    or (location and location in curr_location) \
                    or (pattern and pattern in curr_url):
                    removed += 1

                    if self.config.debug:
                        print('DELETE', self.config.endpoint + 'monitor/' + curr_id)

                    if self.config.readonly:
                        return False

                    # Make request to API endpoint
                    response = requests.delete(self.config.endpoint + 'monitor/' + curr_id, headers=self.config.headers())

                    # Check status code of response
                    if response.status_code == 204:
                        print('Removed site monitor:', curr_url, '[', curr_id, ']')
                        return True
                    else:
                        printError('Failed to remove site monitor', curr_url, '[', curr_id, '] with response code:', response.status_code)
                        return False

        if removed == 0:
            printWarn('No monitors with given pattern found: id=' + id, 'url=', url, 'name=' + name, 'location=' + location, 'pattern=' + pattern)

    def hasIssue(self, monitor):
        """Return True if the specified monitor has some issue by having a value outside of the expected threshold specified in config file"""

        if float(monitor['uptime_percentage']) <= float(self.config.threshold_uptime):
            return True

        if 'last_check' in monitor and 'ttfb' in monitor['last_check']:
            if float(monitor['last_check']['ttfb']) >= float(self.config.threshold_ttfb):
                return True

        return False

    def printHeader(self):
        """Print CSV header if CSV format requested"""
        if (self.format == 'csv'):
            print(self.config.delimiter.join(['ID', 'URL', 'Name', 'Code', 'Status', 'Status Message', 'Uptime %', 'Time to first Byte', 'Location']))

    def printFooter(self, sort: str = '', reverse: bool = False, limit: int = 0):
        """Print table if table format requested"""

        if (self.format == 'table'):

            avg_uptime = self.sum_uptime / self.num_monitors if self.sum_uptime > 0 and self.num_monitors > 0 else 0
            avg_ttfb = self.sum_ttfb / self.num_monitors if self.sum_ttfb > 0 and self.num_monitors > 0 else 0

            if avg_uptime <= float(self.config.threshold_uptime):
                uptime_percentage_text = f"{bcolors.FAIL}" + "{:.4f}".format(avg_uptime) + f"{bcolors.ENDC}"
            else:
                uptime_percentage_text = "{:.4f}".format(avg_uptime)

            if avg_ttfb >= float(self.config.threshold_ttfb):
                ttfb_text = f"{bcolors.FAIL}" + "{:.2f}".format(avg_ttfb) + f"{bcolors.ENDC}"
            else:
                ttfb_text = "{:.2f}".format(avg_ttfb)

            # add average row as table footer
            self.table.add_row(['', 'Average of ' + str(self.num_monitors) + ' monitors', '', uptime_percentage_text, ttfb_text, ''])

            # remove columns that should be excluded
            if self.config.hide_ids:
                self.table.del_column('ID')

            # Get string to be printed and create list of elements separated by \n
            list_of_table_lines = self.table.get_string().split('\n')

            # remember summary row
            summary_line = list_of_table_lines[-2]

            # remove summary row again to allow sorting and limiting
            self.table.del_row(len(self.table.rows)-1)

            if sort:
                # if sort contains the column index instead of the column name, get the column name instead
                if sort.isdecimal():
                    sort = self.table.get_csv_string().split(',')[int(sort) - 1]
            else:
                sort = None

            if limit > 0:
                list_of_table_lines = self.table.get_string(sortby=sort, reversesort=reverse, start=0, end=limit).split('\n')
            else:
                list_of_table_lines = self.table.get_string(sortby=sort, reversesort=reverse).split('\n')

            # Sorting by multiple columns could be done like this
            # list_of_table_lines = self.table.get_string(sortby=("Col Name 1", "Col Name 2")), reversesort=reverse).split('\n')

            # Print the table
            print('\n'.join(list_of_table_lines))
            print(summary_line)
            print(list_of_table_lines[0])

        # elif (self.format == 'csv'):
        #    print(self.table.get_csv_string(delimiter=self.config.delimiter))

    def print(self, monitor):
        """Print the data of the specified web monitor"""

        if (self.format == 'json'):
            print(json.dumps(monitor, indent=4))
            return

        id = monitor['id']
        url = monitor['url']
        name = monitor['name'] if 'name' in monitor else ''
        code = monitor['code'] if 'code' in monitor else ''
        status = monitor['status'] if 'status' in monitor else ''
        status_message = monitor['status_message'] if 'status_message' in monitor else ''
        location = monitor['monitor']['name']
        uptime_percentage = float(monitor['uptime_percentage'])

        if 'last_check' in monitor and 'ttfb' in monitor['last_check']:
            ttfb = float(monitor['last_check']['ttfb'])
            self.sum_uptime = self.sum_uptime + uptime_percentage
            self.sum_ttfb = self.sum_ttfb + ttfb
            self.num_monitors = self.num_monitors + 1
        else:
            ttfb = -1

        if (self.format == 'csv'):
            print(self.config.delimiter.join([id, url, name, str(code), status, status_message, str(uptime_percentage) + '%', str(ttfb), location]))
        else:
            if uptime_percentage <= float(self.config.threshold_uptime):
                uptime_percentage_text = f"{bcolors.FAIL}" + "{:.4f}".format(uptime_percentage) + f"{bcolors.ENDC}"
            else:
                uptime_percentage_text = "{:.4f}".format(uptime_percentage)

            if ttfb >= float(self.config.threshold_ttfb):
                ttfb_text = f"{bcolors.FAIL}" + "{:.2f}".format(ttfb) + f"{bcolors.ENDC}"
            elif ttfb != -1:
                ttfb_text = "{:.2f}".format(ttfb)
            else:
                ttfb_text = f"{bcolors.FAIL}n/a{bcolors.ENDC}"

            self.table.add_row([id, url, status_message, uptime_percentage_text, ttfb_text, location])



#!/usr/bin/env python3

import os
import sys
import setuptools

here = os.path.abspath(os.path.dirname(__file__))

readme = open(os.path.join(here, 'README.md')).read()
install_requires = ['configparser', 'prettytable', 'requests']

setuptools.setup(
    name='360monitoringcli',
    version='1.0.13',
    description='360 Monitoring CLI',
    long_description_content_type='text/markdown',
    long_description=readme,
    url='https://github.com/plesk/360monitoring-cli',
    author='Jan Loeffler',
    author_email='jan.loeffler@webpros.com',
    maintainer='Jan Loeffler',
    maintainer_email='jan.loeffler@webpros.com',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: System :: Monitoring',
    ],
    keywords='360 system monitoring cli',
    install_requires=install_requires,
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            '360monitoring=cli360monitoring.monitoring:main',
        ],
    },
    data_files=[('share/doc/360monitoring-cli', [
        '360monitoring-example.ini',
        'LICENSE',
        'README.md',
    ])],
)



#!/usr/bin/env python3

from prettytable import PrettyTable

from .config import Config
from .servers import Servers

class Recommendations(object):

    def __init__(self, config):
        self.config = config

        self.table = PrettyTable()
        self.table.field_names = ['Asset', 'Recommendation']
        self.table.align['Asset'] = 'l'
        self.table.align['Recommendation'] = 'l'

    def print(self, format: str = 'table'):
        """Iterate through all assets and check for which we have recommendations"""

        servers = Servers(self.config)
        if servers.fetchData():
            for server in servers.servers:
                if servers.hasIssue(server):
                    self.table.add_row([server['name'], servers.getRecommendation(server)])

        if (format == 'table'):
            print(self.table)
        elif (format == 'csv'):
            print(self.table.get_csv_string(delimiter=self.config.delimiter))



#!/usr/bin/env python3

from .bcolors import bcolors

def printError(*args):
    print(f"{bcolors.FAIL}", sep='', end='')
    print(*args, f"{bcolors.ENDC}")

def printWarn(*args):
    print(f"{bcolors.WARNING}", sep='', end='')
    print(*args, f"{bcolors.ENDC}")




#!/usr/bin/env python3

import os
import configparser

from .functions import printError
from .bcolors import bcolors

class Config(object):

    def __init__(self, version: str):
        self.version = version
        self.filename = '360monitoring.ini'
        self.endpoint = 'https://api.monitoring360.io/v1/'
        self.api_key = ''
        self.max_items = 5000
        self.debug = False
        self.readonly = False
        self.hide_ids = False
        self.delimiter = ','

        self.threshold_uptime = 99.0
        self.threshold_ttfb = 1.0
        self.threshold_free_diskspace = 20.0
        self.threshold_cpu_usage = 80.0
        self.threshold_mem_usage = 80.0
        self.threshold_disk_usage = 80.0

        self.loadFromFile()

    def headers(self):
        """Set headers for http requests"""

        if self.api_key:
            return {
                'Authorization': 'Bearer ' + self.api_key,
                'User-Agent': '360 Monitoring CLI ' + self.version,
            }
        else:
            printError('ERROR: No API key specified in ' + self.filename + ". Please run \"360monitoring config save --api-key YOUR_API_KEY\" to connect to your 360 Monitoring account.")
            return {}

    def params(self):
        """Set params for http requests"""

        return {
            'perpage': self.max_items,
            'api_mode': 'cli_' + self.version
        }

    def loadFromFile(self):
        """Read API endpoint and API key from config file"""

        if os.path.isfile(self.filename):
            parser = configparser.ConfigParser()
            parser.read(self.filename)

            if 'Connection' in parser.sections():
                if 'endpoint' in parser['Connection']:
                    self.endpoint = parser['Connection']['endpoint']

                if 'api-key' in parser['Connection']:
                    self.api_key = parser['Connection']['api-key']

                if 'max-items' in parser['Connection']:
                    self.max_items = parser['Connection']['max-items']

                if 'hide-ids' in parser['Connection']:
                    self.hide_ids = (parser['Connection']['hide-ids'] == 'True')

                if 'debug' in parser['Connection']:
                    self.debug = (parser['Connection']['debug'] == 'True')

                if 'readonly' in parser['Connection']:
                    self.readonly = (parser['Connection']['readonly'] == 'True')

            if 'Thresholds' in parser.sections():
                if 'min-uptime-percent' in parser['Thresholds']:
                    self.threshold_uptime = parser['Thresholds']['min-uptime-percent']

                if 'max-time-to-first-byte' in parser['Thresholds']:
                    self.threshold_ttfb = parser['Thresholds']['max-time-to-first-byte']

                if 'min-free-diskspace-percent' in parser['Thresholds']:
                    self.threshold_free_diskspace = parser['Thresholds']['min-free-diskspace-percent']

                if 'max-cpu-usage-percent' in parser['Thresholds']:
                    self.threshold_cpu_usage = parser['Thresholds']['max-cpu-usage-percent']

                if 'max-mem-usage-percent' in parser['Thresholds']:
                    self.threshold_mem_usage = parser['Thresholds']['max-mem-usage-percent']

                if 'max-disk-usage-percent' in parser['Thresholds']:
                    self.threshold_disk_usage = parser['Thresholds']['max-disk-usage-percent']

    def saveToFile(self):
        """Save settings to config file"""

        parser = configparser.ConfigParser()
        parser['Connection'] = {
            'api-key': self.api_key,
            'endpoint': self.endpoint,
            'max-items': self.max_items,
            'hide-ids': self.hide_ids,
            'debug': self.debug,
            'readonly': self.readonly,
        }
        parser['Thresholds'] = {
            'min-uptime-percent': self.threshold_uptime,
            'max-time-to-first-byte': self.threshold_ttfb,
            'min-free-diskspace-percent': self.threshold_free_diskspace,
            'max-cpu-usage-percent': self.threshold_cpu_usage,
            'max-mem-usage-percent': self.threshold_mem_usage,
            'max-disk-usage-percent': self.threshold_disk_usage,
        }

        with open(self.filename, 'w') as config_file:
            parser.write(config_file)

        print('Saved settings to', self.filename)

    def print(self):
        """Print current settings"""

        if os.path.isfile(self.filename):
            print('config file:               ', self.filename)
        else:
            print(f"config file:                {bcolors.WARNING}" + self.filename + f" does not exist. Please run \"360monitoring config save --api-key YOUR_API_KEY\" to configure.{bcolors.ENDC}")
        print('CLI version:               ', self.version)
        print()
        print('Connection')
        print('----------')
        print('endpoint:                  ', self.endpoint)
        if self.api_key:
            print('api-key:                   ', self.api_key)
        else:
            print(f"api-key:                    {bcolors.FAIL}No API key specified in " + self.filename + f". Please run \"360monitoring config save --api-key YOUR_API_KEY\" to connect to your 360 Monitoring account.{bcolors.ENDC}")
        print('max items:                 ', self.max_items)
        print('hide ids:                  ', self.hide_ids)
        print('debug:                     ', self.debug)
        print('readonly:                  ', self.readonly)
        print()
        print('Thresholds')
        print('----------')
        print('min-uptime-percent:        ', self.threshold_uptime)
        print('max-time-to-first-byte:    ', self.threshold_ttfb)
        print('min-free-diskspace-percent:', self.threshold_free_diskspace)
        print('max-cpu-usage-percent:     ', self.threshold_cpu_usage)
        print('max-mem-usage-percent:     ', self.threshold_mem_usage)
        print('max-disk-usage-percent:    ', self.threshold_disk_usage)
